#include "elements.h"
#include <cstdint>


uint16_t Node::nst_id = 0;
uint16_t Finite_element::fst_id = 0;
uint16_t Boundary_Finite_element::bst_id = 0;
Node::Node(uint16_t a, uint16_t b, uint16_t c)
{
	x = a;
	y = b;
	z = c;
	gr = false;
	if (nst_id < 0)
		nst_id = 0;
	nst_id++;
	n_id = nst_id;
}

bool operator <(const Node& a, const Node& b)
{
	if (a.n_id < b.n_id)
		return true;
	else return false;
}

Node::Node()
{
	x = 0;
	y = 0;
	z = 0;
	gr = false;
	if (nst_id < 0)
		nst_id = 0;
	nst_id++;
	n_id = nst_id;
}

Finite_element::Finite_element(uint16_t a, uint16_t b, uint16_t c, uint16_t d, uint16_t mat_id)
{
	nodes[0] = a;
	nodes[1] = b;
	nodes[2] = c;
	nodes[3] = d;
	m_id = mat_id;
	if (fst_id < 0)
		fst_id = 0;
	fst_id++;
	fe_id = fst_id;
}

Finite_element::Finite_element()
{
	for (int i = 0; i < 4; i++)
	{
		nodes[i] = -1;
	}
	m_id = -1;
	if (fst_id < 0)
		fst_id = 0;
	fst_id++;
	fe_id = fst_id;
}

Boundary_Finite_element::Boundary_Finite_element()
{
	g_id = -1;
	for (int i = 0; i < 3; i++)
	{
		nodes[i] = -1;
	}
	if (bst_id < 0)
		bst_id = 0;
	bst_id++;
	bfe_id = bst_id;
}

Boundary_Finite_element::Boundary_Finite_element(uint16_t a, uint16_t b, uint16_t c, uint16_t pov_id)
{
	nodes[0] = a;
	nodes[1] = b;
	nodes[2] = c;
	g_id = pov_id;
	bst_id++;
	bfe_id = bst_id;
}

std::ifstream& operator >>(std::ifstream& in, Node& a)
{
	uint16_t a1, b1, c1;
	in >> a1 >> b1 >> c1;
	a.x = a1;
	a.y = b1;
	a.z = a1;
	return in;
}

std::ifstream& operator >>(std::ifstream& out, Finite_element& a)
{
	uint16_t mat_id, a1, b1, c1, d1;
	out >> mat_id >> a1 >> b1 >> c1 >> d1;
	a.nodes[0] = a1;
	a.nodes[1] = b1;
	a.nodes[2] = c1;
	a.nodes[3] = d1;
	a.m_id = mat_id;
	return out;
}

std::ifstream& operator >>(std::ifstream& in, Boundary_Finite_element& a)
{
	uint16_t b_id, a1, b1, c1;
	in >> b_id >> a1 >> b1 >> c1;
	a.g_id = b_id;
	a.nodes[0] = a1;
	a.nodes[1] = b1;
	a.nodes[2] = c1;
	return in;
}

std::ostream& operator << (std::ostream& out, Node& a)
{
	out << a.x << " " << a.y << " " << a.z << " \n";
	out << "ID : " << a.n_id << " \n";
	return out;
}

std::ostream& operator <<(std::ostream& out, Finite_element& a)
{
	out << " \n";
	for (int i = 0; i < 4; i++)
	{
		out << a.nodes[i];
	}
	out << "ID: " << a.fe_id << "	" << "ID : " << a.m_id << " " << std::endl;

	return out;
}

std::ostream& operator <<(std::ostream& out, Boundary_Finite_element& a)
{
	out << " \n";
	for (int i = 0; i < 3; i++)
	{
		out << a.nodes[i];
	}
	out << "ID: " << a.bfe_id << "ID : " << a.g_id << " " << std::endl;
	return out;
}

bool operator ==(const Node& a, const Node& b)
{
	return (a.n_id == b.n_id);
}